/*
 * devicesettings.c
 *
 * Created: 18.02.2020 22:21:12
 *  Author: Chris
 */ 

#include "usbdrv.h"
#include "devicesettings.h"
#include <avr/wdt.h>
#include <util/delay.h>

ifmode_t EEMEM ee_ifmode = ifmode_pad;
uint8_t EEMEM ee_key_keycode[2];
uint8_t EEMEM ee_kbseq[KBSEQ_LENGTH] = {};

ds_respdata_t ds_response;
ifmode_t interface_mode = ifmode_pad;
uint8_t vt_key_keycode[2] = { 0x00, 0x00 };
uint8_t vt_kbseq[KBSEQ_LENGTH];

static inline void reset(void)
{
    usbDeviceDisconnect();
    for(uint8_t i = 0; i < 25; i++)
    {
        wdt_reset();
        _delay_ms(10);
    }
    wdt_enable(WDTO_1S);
    while(true);

}

void ds_cmdReceived(uchar* buffer)
{
    if(ds_response.dataready == true)
    {
        return;
    }

    dscmd_t cmd = buffer[0];
    if(cmd == dscmd_restart)
    {
        reset();
    }
    else if(cmd == dscmd_setifmode)
    {
        eeprom_write_byte(&ee_ifmode, buffer[1]);
        ds_response.data[0] = cmd | dsresp_ok;
        ds_response.dataready = true;
    }
    else if(cmd == dscmd_getifmode)
    {
        ds_response.data[0] = cmd | dsresp_ok;
        ds_response.data[1] = eeprom_read_byte(&ee_ifmode);
        ds_response.dataready = true;

    }
    else if(cmd == dscmd_setkbdkey)
    {
        eeprom_write_byte(&ee_key_keycode[0], buffer[1]);
        eeprom_write_byte(&ee_key_keycode[1], buffer[2]);
        
        ds_response.data[0] = cmd | dsresp_ok;
        ds_response.dataready = true;
    }
    else if(cmd == dscmd_getkbdkey)
    {
        ds_response.data[0] = cmd | dsresp_ok;
        ds_response.data[1] = eeprom_read_byte(&ee_key_keycode[0]);
        ds_response.data[2] = eeprom_read_byte(&ee_key_keycode[1]);
        ds_response.dataready = true;
    }
    else if(cmd == dscmd_setkbseq)
    {
        uint8_t chunk = buffer[1];
        uint8_t chunkoffset = chunk * 6;
        uint8_t chunklength = 6;
        if(chunkoffset >= KBSEQ_LENGTH)
        {
            ds_response.data[0] = cmd | dsresp_error;
            ds_response.dataready = true;
            return;
        }
        if(KBSEQ_LENGTH - chunkoffset < chunklength)
        {
            chunklength = KBSEQ_LENGTH - chunkoffset;
        }
        eeprom_write_block(&buffer[2], ee_kbseq + chunkoffset, chunklength);
        ds_response.data[0] = cmd | dsresp_ok;
        ds_response.dataready = true;
    }
    else if(cmd == dscmd_getkbseq)
    {
        uint8_t chunk = buffer[1];
        uint8_t chunkoffset = chunk * 6;
        uint8_t chunklength = 6;
        if(chunkoffset >= KBSEQ_LENGTH)
        {
            ds_response.data[0] = cmd | dsresp_error;
            ds_response.dataready = true;
            return;
        }
        if(KBSEQ_LENGTH - chunkoffset < chunklength)
        {
            chunklength = KBSEQ_LENGTH - chunkoffset;
        }
        eeprom_read_block(&ds_response.data[2], ee_kbseq + chunkoffset, chunklength);
        ds_response.data[0] = cmd | dsresp_ok;
        ds_response.data[1] = chunk;
        ds_response.dataready = true;
    }
}

void ds_loadifmode(void)
{
    interface_mode = eeprom_read_byte(&ee_ifmode);
    if(interface_mode > INTERFACE_MODE_MAX)
        interface_mode = ifmode_hid;
}

void ds_loadkbdkey(void)
{
    eeprom_read_block(vt_key_keycode, ee_key_keycode, sizeof(ee_key_keycode));
}

void ds_loadkbseq(void)
{
    eeprom_read_block(vt_kbseq, ee_kbseq, sizeof(ee_kbseq));
}

dsresp_t ds_task(void)
{
    
    return dsresp_none;
}