/*
 * keyboard.h
 *
 * Created: 12.02.2020 11:26:52
 *  Author: Chris
 */ 


#ifndef KEYBOARD_H_
#define KEYBOARD_H_

#include <stdbool.h>
#include <avr/io.h>

/* Keyboard usage values, see usb.org's HID-usage-tables document, chapter
 * 10 Keyboard/Keypad Page for more codes.
 */

#define MOD_CONTROL_LEFT    (1<<0)
#define MOD_SHIFT_LEFT      (1<<1)
#define MOD_ALT_LEFT        (1<<2)
#define MOD_GUI_LEFT        (1<<3)
#define MOD_CONTROL_RIGHT   (1<<4)
#define MOD_SHIFT_RIGHT     (1<<5)
#define MOD_ALT_RIGHT       (1<<6)
#define MOD_GUI_RIGHT       (1<<7)

#define KEY_A       0x04
#define KEY_B       0x05
#define KEY_C       0x06
#define KEY_D       0x07
#define KEY_E       0x08
#define KEY_F       0x09
#define KEY_G       0x0A
#define KEY_H       0x0B
#define KEY_I       0x0C
#define KEY_J       0x0D
#define KEY_K       0x0E
#define KEY_L       0x0F
#define KEY_M       0x10
#define KEY_N       0x11
#define KEY_O       0x12
#define KEY_P       0x13
#define KEY_Q       0x14
#define KEY_R       0x15
#define KEY_S       0x16
#define KEY_T       0x17
#define KEY_U       0x18
#define KEY_V       0x19
#define KEY_W       0x1A
#define KEY_X       0x1B
#define KEY_Y       0x1C
#define KEY_Z       0x1D
#define KEY_1       0x1E
#define KEY_2       0x1F
#define KEY_3       0x20
#define KEY_4       0x21
#define KEY_5       0x22
#define KEY_6       0x23
#define KEY_7       0x24
#define KEY_8       0x25
#define KEY_9       0x26
#define KEY_0       0x27
#define KEY_RETURN  0x28
#define KEY_ESCAPE  0x29
#define KEY_DELETE  0x2A
#define KEY_TAB     0x2B
#define KEY_SPACE   0x2C

#define KEY_F1      58
#define KEY_F2      59
#define KEY_F3      60
#define KEY_F4      61
#define KEY_F5      62
#define KEY_F6      63
#define KEY_F7      64
#define KEY_F8      65
#define KEY_F9      66
#define KEY_F10     67
#define KEY_F11     68
#define KEY_F12     69

typedef struct
{
    bool running;
    bool release_mod;
    bool release_key;
    bool blank;
    uint8_t position;
    uint8_t modifiers;
    uint8_t last_keycode;
    uint8_t keycode;
    uint8_t pause;
} kbseqstate_t;

void keyboard_init(void);
void keyboard_trigger(void);
void keyboard_tick(void);
bool keyboard_process(void);

extern kbseqstate_t kbseq;

#endif /* KEYBOARD_H_ */