/*
 * usbdescriptors.h
 *
 * Created: 14.02.2020 21:06:54
 *  Author: Chris
 */ 


#ifndef USBDESCRIPTORS_H_
#define USBDESCRIPTORS_H_

#include <avr/pgmspace.h>

// low byte first
#define USB_VENDOR_ID_HID 0xC0, 0x16
#define USB_DEVICE_ID_HID 0xD9, 0x27

#define USB_VENDOR_ID_KB 0xC0, 0x16
#define USB_DEVICE_ID_KB 0xDB, 0x27

#define USB_VENDOR_ID_PAD 0xC0, 0x16
#define USB_DEVICE_ID_PAD 0xDC, 0x27

typedef struct
{
    uint16_t* data;
    uint8_t size;
}   usbdescrentry_t;

typedef struct
{
    usbdescrentry_t device;
    usbdescrentry_t config;
    usbdescrentry_t devname;
    usbdescrentry_t hid;
    usbdescrentry_t hidreport;
}   usbdescr_t;


#define usb_cfg_device_vendorname_LEN 19
static PROGMEM const int usb_cfg_device_vendorname[usb_cfg_device_vendorname_LEN + 1] = {
    USB_STRING_DESCRIPTOR_HEADER( usb_cfg_device_vendorname_LEN ),
    'h', 'o', 'b', 'b', 'y', 'e', 'l', 'e', 'k', 't', 'r', 'o', 'n', 'i', 'k', '.', 'o', 'r', 'g'
};

//obligatory for shared VID/PID usage
#define usb_cfg_device_serialnumber_LEN 30
static PROGMEM const int usb_cfg_device_serialnumber[usb_cfg_device_serialnumber_LEN + 1] = {
    USB_STRING_DESCRIPTOR_HEADER( usb_cfg_device_serialnumber_LEN ),
    'h', 'o', 'b', 'b', 'y', 'e', 'l', 'e', 'k', 't', 'r', 'o', 'n', 'i', 'k', '.', 'o', 'r', 'g', ':', 'F', 'o', 'o', 't', 's', 'w', 'i', 't', 'c', 'h'
};

PROGMEM const char usb_cfg_device_descriptor_hid[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescriptorDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,        /* descriptor type */
    0x10, 0x01,             /* USB version supported */
    USB_CFG_DEVICE_CLASS,
    USB_CFG_DEVICE_SUBCLASS,
    0,                      /* protocol */
    8,                      /* max packet size */
    /* the following two casts affect the first byte of the constant only, but
     * that's sufficient to avoid a warning with the default values.
     */
    (char)USB_VENDOR_ID_HID,/* 2 bytes */
    (char)USB_DEVICE_ID_HID,/* 2 bytes */
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
    USB_CFG_DESCR_PROPS_STRING_VENDOR != 0 ? 1 : 0,         /* manufacturer string index */
    USB_CFG_DESCR_PROPS_STRING_PRODUCT != 0 ? 2 : 0,        /* product string index */
    USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER != 0 ? 3 : 0,  /* serial number string index */
    1,          /* number of configurations */
};

#define usb_cfg_device_name_hid_LEN 14
static PROGMEM const int usb_cfg_device_name_hid[usb_cfg_device_name_hid_LEN + 1] = {
    USB_STRING_DESCRIPTOR_HEADER( usb_cfg_device_name_hid_LEN ),
    'F', 'o', 'o', 't', 's', 'w', 'i', 't', 'c', 'h', ' ', 'H', 'I', 'D'
};

#define usbHidReportDescriptor_hid_LEN 32
const PROGMEM char usbHidReportDescriptor_hid[usbHidReportDescriptor_hid_LEN] = /* USB report descriptor */
{
    0x06, 0x00, 0xFF,  // Usage Page (Vendor Defined 0xFF00)
    0x09, 0x01,        // Usage (0x01)
    0xA1, 0x01,        // Collection (Application)
    0x09, 0x02,        //   Usage (0x02)
    0x15, 0x00,        //   Logical Minimum (0)
    0x25, 0xFF,        //   Logical Maximum (-1)
    0x75, 0x08,        //   Report Size (8)
    0x95, 0x08,        //   Report Count (8)
    0x81, 0x02,        //   Input (Data,Var,Abs,No Wrap,Linear,Preferred State,No Null Position)
    0x09, 0x03,        //   Usage (0x03)
    0x15, 0x00,        //   Logical Minimum (0)
    0x25, 0xFF,        //   Logical Maximum (-1)
    0x75, 0x08,        //   Report Size (8)
    0x95, 0x08,        //   Report Count (8)
    0x91, 0x02,        //   Output (Data,Var,Abs,No Wrap,Linear,Preferred State,No Null Position,Non-volatile)
    0xC0,              // End Collection
};

static PROGMEM const char usbDescriptorConfiguration_hid[] = {    /* USB configuration descriptor */
    9,          /* sizeof(usbDescriptorConfiguration): length of descriptor in bytes */
    USBDESCR_CONFIG,    /* descriptor type */
    //18 + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT3 + (USB_CFG_DESCR_PROPS_HID & 0xff), 0,
    //         ^^ 1x                              ^^ 0                             1
    // sums up to:
    18+7+9, 0,
    /* total length of data returned (including inlined descriptors) */
    1,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
    #if USB_CFG_IS_SELF_POWERED
    (1 << 7) | USBATTR_SELFPOWER,       /* attributes */
    #else
    (1 << 7),                           /* attributes */
    #endif
    USB_CFG_MAX_BUS_POWER/2,            /* max USB current in 2mA units */
    /* interface descriptor follows inline: */
    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    USB_CFG_HAVE_INTRIN_ENDPOINT + USB_CFG_HAVE_INTRIN_ENDPOINT3, /* endpoints excl. 0: number of endpoint descriptors to follow */
    USB_CFG_INTERFACE_CLASS,
    USB_CFG_INTERFACE_SUBCLASS,
    USB_CFG_INTERFACE_PROTOCOL,
    0,          /* string index for interface */
    // here begins the "embedded" USB_CFG_DESCR_PROPS_HID description
    9,          /* sizeof(usbDescrHID): length of descriptor in bytes */
    USBDESCR_HID,   /* descriptor type: HID */
    0x01, 0x01, /* BCD representation of HID version */
    0x00,       /* target country code */
    0x01,       /* number of HID Report (or other HID class) Descriptor infos to follow */
    0x22,       /* descriptor type: report */
    usbHidReportDescriptor_hid_LEN, 0,  /* total length of report descriptor */
    // here ends the "embedded" USB_CFG_DESCR_PROPS_HID description
    #if USB_CFG_HAVE_INTRIN_ENDPOINT    /* endpoint descriptor for endpoint 1 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)0x81, /* IN endpoint number 1 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
    #endif
    #if USB_CFG_HAVE_INTRIN_ENDPOINT3   /* endpoint descriptor for endpoint 3 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)(0x80 | USB_CFG_EP3_NUMBER), /* IN endpoint number 3 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
    #endif
};


PROGMEM const char usb_cfg_device_descriptor_kb[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescriptorDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,        /* descriptor type */
    0x10, 0x01,             /* USB version supported */
    USB_CFG_DEVICE_CLASS,
    USB_CFG_DEVICE_SUBCLASS,
    0,                      /* protocol */
    8,                      /* max packet size */
    /* the following two casts affect the first byte of the constant only, but
     * that's sufficient to avoid a warning with the default values.
     */
    (char)USB_VENDOR_ID_KB,/* 2 bytes */
    (char)USB_DEVICE_ID_KB,/* 2 bytes */
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
    USB_CFG_DESCR_PROPS_STRING_VENDOR != 0 ? 1 : 0,         /* manufacturer string index */
    USB_CFG_DESCR_PROPS_STRING_PRODUCT != 0 ? 2 : 0,        /* product string index */
    USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER != 0 ? 3 : 0,  /* serial number string index */
    1,          /* number of configurations */
};


#define usbHidReportDescriptor_kb_LEN 35
const PROGMEM char usbHidReportDescriptor_kb[usbHidReportDescriptor_kb_LEN] = /* USB report descriptor */
{   
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
/* We use a simplified keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */

 

 #define usb_cfg_device_name_kb_LEN 13
 static PROGMEM const int usb_cfg_device_name_kb[usb_cfg_device_name_kb_LEN+1] = {
     USB_STRING_DESCRIPTOR_HEADER( usb_cfg_device_name_kb_LEN ),
     'F', 'o', 'o', 't', 's', 'w', 'i', 't', 'c', 'h',' ', 'K', 'B'
 };



static PROGMEM const char usbDescriptorConfiguration_kb[] = {    /* USB configuration descriptor */
    9,          /* sizeof(usbDescriptorConfiguration): length of descriptor in bytes */
    USBDESCR_CONFIG,    /* descriptor type */
    //18 + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT3 + (USB_CFG_DESCR_PROPS_HID & 0xff), 0,
    //         ^^ 1x                              ^^ 0                             1
    // sums up to:
    18+7+9, 0,

    /* total length of data returned (including inlined descriptors) */
    1,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
    #if USB_CFG_IS_SELF_POWERED
    (1 << 7) | USBATTR_SELFPOWER,       /* attributes */
    #else
    (1 << 7),                           /* attributes */
    #endif
    USB_CFG_MAX_BUS_POWER/2,            /* max USB current in 2mA units */
    /* interface descriptor follows inline: */
    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    USB_CFG_HAVE_INTRIN_ENDPOINT + USB_CFG_HAVE_INTRIN_ENDPOINT3, /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CFG_INTERFACE_CLASS,
    USB_CFG_INTERFACE_SUBCLASS,
    USB_CFG_INTERFACE_PROTOCOL,
    0,          /* string index for interface */
    
    9,          /* sizeof(usbDescrHID): length of descriptor in bytes */
    USBDESCR_HID,   /* descriptor type: HID */
    0x01, 0x01, /* BCD representation of HID version */
    0x00,       /* target country code */
    0x01,       /* number of HID Report (or other HID class) Descriptor infos to follow */
    0x22,       /* descriptor type: report */
    usbHidReportDescriptor_kb_LEN, 0,  /* total length of report descriptor */
    
    #if USB_CFG_HAVE_INTRIN_ENDPOINT    /* endpoint descriptor for endpoint 1 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)0x81, /* IN endpoint number 1 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
    #endif
    #if USB_CFG_HAVE_INTRIN_ENDPOINT3   /* endpoint descriptor for endpoint 3 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)(0x80 | USB_CFG_EP3_NUMBER), /* IN endpoint number 3 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
    #endif
};


PROGMEM const char usb_cfg_device_descriptor_pad[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescriptorDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,        /* descriptor type */
    0x10, 0x01,             /* USB version supported */
    USB_CFG_DEVICE_CLASS,
    USB_CFG_DEVICE_SUBCLASS,
    0,                      /* protocol */
    8,                      /* max packet size */
    /* the following two casts affect the first byte of the constant only, but
     * that's sufficient to avoid a warning with the default values.
     */
    (char)USB_VENDOR_ID_PAD,/* 2 bytes */
    (char)USB_DEVICE_ID_PAD,/* 2 bytes */
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
    USB_CFG_DESCR_PROPS_STRING_VENDOR != 0 ? 1 : 0,         /* manufacturer string index */
    USB_CFG_DESCR_PROPS_STRING_PRODUCT != 0 ? 2 : 0,        /* product string index */
    USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER != 0 ? 3 : 0,  /* serial number string index */
    1,          /* number of configurations */
};



#define usb_cfg_device_name_pad_LEN 14
static PROGMEM const int usb_cfg_device_name_pad[usb_cfg_device_name_pad_LEN + 1] = {
    USB_STRING_DESCRIPTOR_HEADER( usb_cfg_device_name_pad_LEN ),
    'F', 'o', 'o', 't', 's', 'w', 'i', 't', 'c', 'h', ' ', 'P', 'a', 'd'
};


//it works but still it is a little sketchy
#define usbHidReportDescriptor_pad_LEN 28
const PROGMEM char usbHidReportDescriptor_pad[usbHidReportDescriptor_pad_LEN] = {
    0x05, 0x01,        // Usage Page (Generic Desktop Ctrls)
    0x09, 0x04,        // Usage (Joystick)
    0xA1, 0x01,        // Collection (Application)
    0x09, 0x04,        //   Usage (Joystick)
    0xA1, 0x00,        //   Collection (Physical)
    0x05, 0x09,        //     Usage Page (Button)
    0x19, 0x01,        //     Usage Minimum (0x01)
    0x29, 0x01,        //     Usage Maximum (0x01)
    0x15, 0x00,        //     Logical Minimum (0)
    0x25, 0x01,        //     Logical Maximum (1)
    0x95, 0x08,        //     Report Count (8)
    0x75, 0x01,        //     Report Size (1)
    0x81, 0x02,        //     Input (Data,Var,Abs,No Wrap,Linear,Preferred State,No Null Position)
    0xC0,              //   End Collection
    0xC0,              // End Collection
};

static PROGMEM const char usbDescriptorConfiguration_pad[] = {    /* USB configuration descriptor */
    9,          /* sizeof(usbDescriptorConfiguration): length of descriptor in bytes */
    USBDESCR_CONFIG,    /* descriptor type */
    //18 + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT3 + (USB_CFG_DESCR_PROPS_HID & 0xff), 0,
    //         ^^ 1x                              ^^ 0                             1
    // sums up to:
    18+7+9, 0,
    /* total length of data returned (including inlined descriptors) */
    1,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
    #if USB_CFG_IS_SELF_POWERED
    (1 << 7) | USBATTR_SELFPOWER,       /* attributes */
    #else
    (1 << 7),                           /* attributes */
    #endif
    USB_CFG_MAX_BUS_POWER/2,            /* max USB current in 2mA units */
    /* interface descriptor follows inline: */
    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    USB_CFG_HAVE_INTRIN_ENDPOINT + USB_CFG_HAVE_INTRIN_ENDPOINT3, /* endpoints excl. 0: number of endpoint descriptors to follow */
    USB_CFG_INTERFACE_CLASS,
    USB_CFG_INTERFACE_SUBCLASS,
    USB_CFG_INTERFACE_PROTOCOL,
    0,          /* string index for interface */
    // here begins the "embedded" USB_CFG_DESCR_PROPS_HID description
    9,          /* sizeof(usbDescrHID): length of descriptor in bytes */
    USBDESCR_HID,   /* descriptor type: HID */
    0x01, 0x01, /* BCD representation of HID version */
    0x00,       /* target country code */
    0x01,       /* number of HID Report (or other HID class) Descriptor infos to follow */
    0x22,       /* descriptor type: report */
    usbHidReportDescriptor_pad_LEN, 0,  /* total length of report descriptor */
    // here ends the "embedded" USB_CFG_DESCR_PROPS_HID description
    #if USB_CFG_HAVE_INTRIN_ENDPOINT    /* endpoint descriptor for endpoint 1 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)0x81, /* IN endpoint number 1 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
    #endif
    #if USB_CFG_HAVE_INTRIN_ENDPOINT3   /* endpoint descriptor for endpoint 3 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)(0x80 | USB_CFG_EP3_NUMBER), /* IN endpoint number 3 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL, /* in ms */
    #endif
};

#define USBDESCRENTRY_REF(x) { \
    .data = (uint16_t*)x, \
    .size = sizeof(x), \
}

//TODO: move to PROGMEM
usbdescr_t usbdescr_kb = {
    .device = USBDESCRENTRY_REF(usb_cfg_device_descriptor_kb),
    .config = USBDESCRENTRY_REF(usbDescriptorConfiguration_kb),
    .devname = USBDESCRENTRY_REF(usb_cfg_device_name_kb),

    .hid = {
        .data = (uint16_t*)(usbDescriptorConfiguration_kb + 9 + 9),
        .size = 9,
    },

    .hidreport = USBDESCRENTRY_REF(usbHidReportDescriptor_kb),
};

usbdescr_t usbdescr_pad = {
    .device = USBDESCRENTRY_REF(usb_cfg_device_descriptor_pad),
    .config = USBDESCRENTRY_REF(usbDescriptorConfiguration_pad),
    .devname = USBDESCRENTRY_REF(usb_cfg_device_name_pad),

    .hid = {
        .data = (uint16_t*)(usbDescriptorConfiguration_pad + 9 + 9),
        .size = 9,
    },

    .hidreport = USBDESCRENTRY_REF(usbHidReportDescriptor_pad),
};

usbdescr_t usbdescr_hid = {
    .device = USBDESCRENTRY_REF(usb_cfg_device_descriptor_hid),
    .config = USBDESCRENTRY_REF(usbDescriptorConfiguration_hid),
    .devname = USBDESCRENTRY_REF(usb_cfg_device_name_hid),

    .hid = {
        .data = (uint16_t*)(usbDescriptorConfiguration_hid + 9 + 9),
        .size = 9,
    },

    .hidreport = USBDESCRENTRY_REF(usbHidReportDescriptor_hid),
};


#endif /* USBDESCRIPTORS_H_ */